/*
 *  Box.cpp
 *  visualizer2
 *
 *  Created by Mark Watson on Tue Apr 06 2004.
 *  Copyright (c) 2004 University of Saskatchewan. All rights reserved.
 *
 */


#ifdef WIN32
#include <gl\glut.h>
#else
#include <GLUT/glut.h>
#endif

#include <iostream>
#include <cstdlib>
#include <vector>
#include "Primaries.h"
#include "Box.h"
#include "Text.h"

using namespace std;

#define XD 700
#define YD 700
#define ZD 500
#define PINS 200
float movement = 0;
int changed = 0;
int number_of_pins = 0;
int number_of_selected = 0;
int cur_selected = 0;
vector<Pin*> pins( PINS );
Pin* selected[100];
int mode = 0;
int num_modes = 5;
int view_mode = 0;
int num_views = 3;


// creates the scene
void setup(void){
	movement = 0;
	changed = 0;
	number_of_pins = 0;
	number_of_selected = 0;
	cur_selected = 0;
	if(mode == 0){
		Box* child = new Box(1., 1., 1.);
		Pin* childp = new Pin(0.,0.,0., child);
		Box* b1 = new Box(3., 3., 3.);
		b1->add_to_face(childp, 1);
		b1->add_to_face(childp, 2);
		b1->add_to_face(childp, 3);
		b1->add_to_face(childp, 4);
		b1->add_to_face(childp, 5);
		b1->add_to_face(childp, 6);
		//Pin* p1 = new Pin(0., 0., -10., 30., 30., 30., b1);
		Pin* p1 = new Pin(0., 0., -10., b1);
		pins[0] = p1;
		number_of_pins++;
		Box* b2 = new Box(2., 2., 2.);
		Pin* p2 = new Pin(-10., 0., -15., 30., 30., 90., b2);
		pins[1] = p2;
		number_of_pins++;
	}
	else if(mode == 1){
		Box* b1 = new Box(3., 3., 3.);
		Pin* p1 = new Pin(-10., 0., -15., 30., 30., 90., b1); 
		pins[0] = p1;
		number_of_pins++;
		Box* b2 = new Box(1., 1., 1.);
		Pin* p2 = new Pin(0., 0., -.3, b2);
		pins[1] = p2;
		number_of_pins++;
	}
	else if(mode == 2){
		Box* b = new Box(5., 5., 5.);
		Pin* p = new Pin(0., 0., -15., 45., 45., 0., b); 
		pins[0] = p;
		number_of_pins++;
	}
	else if(mode == 3){
		Text* t = new Text("This is Text");
		Pin* p = new Pin(0., 0., -15., t); 
		pins[0] = p;
		number_of_pins++;
	}
	else if(mode == 4){
		Box* b1 = new Box(5., 5., 5.);
		Pin* p1 = new Pin(-10., 0., -15., b1); 
		pins[0] = p1;
		number_of_pins++;
		Text* t = new Text("This is Text");
		Pin* p = new Pin(0., -5.1, 1., 180, 0, -90, t); 
		b1->add_to_face(p, 1);
	}
}


// the main drawing algorithm
void draw_gels(void){
	glLoadIdentity();									// Reset The Current Modelview Matrix
	for( int i = 0; i < number_of_pins ; i++){
		glPushMatrix();
		glTranslatef(pins[i]->get_x(),pins[i]->get_y(),pins[i]->get_z());
		glRotatef( pins[i]->get_roll(), 1., 0., 0.);
		glRotatef( pins[i]->get_pitch(), 0., 1., 0.);
		glRotatef( pins[i]->get_yaw(), 0., 0., 1.);
		pins[i]->get_gel()->draw_self();
		glPopMatrix();
	}
}

// CORE GL STUFF

// sets up the display
void display(void) {
    glLoadIdentity();
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glCallList(1);
	draw_gels();
	//b.draw_self(0,0,);
	//movement = movement + 0.01;
    glutSwapBuffers();
}

// INPUT


// selects the next item
void select_next(void){
	if(number_of_pins){
		cur_selected = (cur_selected + 1) % number_of_pins;
		number_of_selected = 1;
		selected[0] = pins[cur_selected];
	}
	else {
		number_of_selected = 0;
	}
}

// selects the previous item
void select_prev(void){
	if(number_of_pins){
		cur_selected = (cur_selected - 1) % number_of_pins;
		cur_selected > 0 ? cur_selected = number_of_pins - 1 : cur_selected = cur_selected;  
		number_of_selected = 1;
		selected[0] = pins[cur_selected];
	}
	else {
		number_of_selected = 0;
	}
}


//  Position

//moves a pin forward
void forward_pin(void) {
	if(number_of_selected){
		selected[0]->set_z(selected[0]->get_z()+0.5);
	}
}

// moves a pin back
void back_pin(void) {
	if(number_of_selected){
		selected[0]->set_z(selected[0]->get_z()-0.5);
	}
}

// moves a pin upward
void up_pin(void) {
	if(number_of_selected){
		selected[0]->set_y(selected[0]->get_y()+0.5);
	}
}

// moves a pin downward
void down_pin(void) {
	if(number_of_selected){
		selected[0]->set_y(selected[0]->get_y()-0.5);
	}
}

// moves a pin right
void right_pin(void) {
	if(number_of_selected){
		selected[0]->set_x(selected[0]->get_x()+0.5);
	}
}

// moves a pin left
void left_pin(void) {
	if(number_of_selected){
		selected[0]->set_x(selected[0]->get_x()-0.5);
	}
}

// ORIENTATION

// roll a pin 2.5 degrees
void roll_pin(void) {
	if(number_of_selected){
		selected[0]->set_roll(selected[0]->get_roll()+2.5);
	}
}

// roll a pin -2.5 degrees
void roll_pin_back(void) {
	if(number_of_selected){
		selected[0]->set_roll(selected[0]->get_roll()-2.5);
	}
}

// increase a pin's yaw by 2.5 degrees
void yaw_pin(void) {
	if(number_of_selected){
		selected[0]->set_yaw(selected[0]->get_yaw()+2.5);
	}
}

// decrease a pin's yaw by 2.5 degrees
void yaw_pin_back(void) {
	if(number_of_selected){
		selected[0]->set_yaw(selected[0]->get_yaw()-2.5);
	}
}

// increase a pin's pitch by 2.5 degrees
void pitch_pin(void) {
	if(number_of_selected){
		selected[0]->set_pitch(selected[0]->get_pitch()+2.5);
	}
}

// decrease a pin's pitch by 2.5 degrees
void pitch_pin_back(void) {
	if(number_of_selected){
		selected[0]->set_pitch(selected[0]->get_pitch()-2.5);
	}
}

// swap two pins
void swap_pins(){
	for( int i = 0; i+1 < number_of_pins ; i++){
		Gel* g = pins[i]->get_gel(); 
		pins[i]->set_gel(pins[i+1]->get_gel());
		pins[i+1]->set_gel(g);
	}
}

// stick one gel onto another
void compose_pins() {
	if(number_of_pins > 1) { 
		number_of_pins--;
		pins[0]->get_gel()->add_to_face(pins[1], 1);
	}
}

// change the view setup
void swap_view() {
		if(view_mode == 0) {
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(90.0, ((float) XD )/ ((float) YD), 0.1, 100.0);
			//glFrustum(0., 50., 0., 50., 1., 100.);
			glShadeModel(GL_FLAT);
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
		}
		else if(view_mode == 1){
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			glOrtho(-10., 10., -10., 10., .1, 100.);
			glShadeModel(GL_FLAT);
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
		}
		else if(view_mode == 2){
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			glFrustum(-10., 10., -10., 10., 10., 100.);
			glShadeModel(GL_FLAT);
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
		}

}

// take keyboard commands
void keyboard(unsigned char key, int x, int y) {
	if(key == 'n') { //select next Gel
		select_next();
	}
	else if(key == 'N') { //add next Gel
		//add_next();
	}
	else if(key == 'p') { //select previous Gel
		//select_prev();
	}
	else if(key == 'r') {
		roll_pin();
	}
	else if(key == 'R') {
		roll_pin_back();
	}
	else if(key == 't') {
		pitch_pin();
	}
	else if(key == 'T') {
		pitch_pin_back();
	}
	else if(key == 'y') {
		yaw_pin();
	}
	else if(key == 'Y') {
		yaw_pin_back();
	}
	else if(key == 's') {
		swap_pins();
	}
	else if (key == 'c') {
		compose_pins();
	}
	else if (key == 'v') {
		view_mode = (view_mode + 1) % num_views;
		swap_view();
	}
	else if(key == '8') {
		up_pin();
	}
	else if(key == '2') {
		down_pin();
	}
	else if(key == '4') {
		left_pin();
	}
	else if(key == '6') {
		right_pin();
	}
	else if(key == '+') {
		forward_pin();
	}
	else if(key == '-') {
		back_pin();
	}	
	else if(key == '`') {
		mode = (mode + 1) % num_modes;
		setup();
	}	
    else if (key == 'q') {
        exit(0);
    }
	else{
		std::cout << "the key stroke is: ";
		std::cout << key;
		std::cout << "\n";
	}
	changed = 1;
}

void mousebutton(int btn, int state, int x, int y) {
    if (state == GLUT_DOWN) {
    }
    
    if (state == GLUT_UP) {
    }
	changed = 1;   
}

void mousemoved(int x, int y){
	changed = 1;
}


// if nothing has changed, do nothing, else redraw
void idle(void) {
    if(changed){
        glutPostRedisplay();
		changed = 0;
	}
}

// setup the general properties of the world
void set_world(void) {
	int window;
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	window = glutCreateWindow("AUI3D");
    glutReshapeWindow(XD,YD);
    glutSetWindow(window);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(90.0, ((float) XD )/ ((float) YD), 0.1, 100.0);
	//glFrustum(0., 50., 0., 50., 1., 100.);
    glShadeModel(GL_FLAT);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	
    glClearColor(1, 1, 1, 1);
    changed = 1;
	glDepthFunc(GL_LESS);						// The Type Of Depth Test To Do
	glEnable(GL_DEPTH_TEST);					// Enables Depth Testing 
	
    glutKeyboardFunc(keyboard);
    glutMouseFunc(mousebutton);
    glutMotionFunc(mousemoved);
    glutDisplayFunc(display);
    glutIdleFunc(idle);
	
}

// start the program
int main(int argc, char* argv[]) {
    // using namespace std;
	glutInit(&argc, argv);
	set_world();
	setup();
    glutMainLoop();
    return 0;
}
